require 'github_api'
require 'rugged'

module Github
  class Repository
    InvalidRepositoryDirectory = Class.new(ArgumentError)
    MissingRemoteOrigin = Class.new(RuntimeError)
    InvalidRemoteOrigin = Class.new(RuntimeError)

    attr_reader :host, :owner, :name

    def initialize(dir)
      @repository = Rugged::Repository.new(dir)

      origin = @repository.remotes['origin']
      raise MissingRemoteOrigin unless origin

      matches = origin.url.match(/\A(git@|https:\/\/)([^\/:]+)(\/|:)([^\/:]+)\/([^\/]+)\.git\z/)
      unless matches
        raise InvalidRemoteOrigin, 'Invalid remote url'
      end

      @host, @owner, @name = matches[2], matches[4], matches[5]
      unless Github::CLI.config.keys.include?(@host)
        raise InvalidRemoteOrigin, 'Unrecognized git host'
      end
    rescue Rugged::RepositoryError
      raise InvalidRepositoryDirectory
    end

    def id
      "#{owner}/#{name}"
    end

    def issues(state: 'open', sort: 'updated', direction: 'desc', page: 1, per_page: 10)
      client.issues.list(
        user: owner,
        repo: name,
        state: state,
        sort: sort,
        direction: direction,
        page: page,
        per_page: per_page
      )
    end

    def issue(number)
      client.issues.get(
        user: owner,
        repo: name,
        number: number
      )
    end

    def issue_comments(number)
      client.issues.comments.list(
        user: owner,
        repo: name,
        number: number
      )
    end

    def pull_requests(state: 'open', sort: 'updated', direction: 'desc', page: 1, per_page: 10)
      client.pull_requests.list(
        user: owner,
        repo: name,
        state: state,
        sort: sort,
        direction: direction,
        page: page,
        per_page: per_page
      )
    end

    def pull_request(number)
      client.pull_requests.get(
        user: owner,
        repo: name,
        number: number
      )
    end

    def pull_request_commits(number)
      client.pull_requests.commits(
        user: owner,
        repo: name,
        number: number
      )
    end

    def pull_request_files(number)
      client.pull_requests.files(
        user: owner,
        repo: name,
        number: number
      )
    end

    def pull_request_comments(number)
      client.pull_requests.comments(
        user: owner,
        repo: name,
        number: number
      )
    end

    private

    attr_reader :repository

    def client
      @client ||= Github.new do |c|
        c.oauth_token = Github::CLI.config[host]['access_token']
        c.endpoint = "https://#{host}/api/v3/" if host != 'github.com'
      end
    end
  end
end
